﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Devices.Sensors;
using Microsoft.Xna.Framework;
using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Controls.Maps.Core;
using System.Device.Location;


namespace PhoneAppMaps
{
    public partial class MainPage : PhoneApplicationPage
    {
        // The GPS sensor
        private GeoCoordinateWatcher gps = null;

        // Our current gps location
        public GeoCoordinate mostRecentPosition { get; set; }

        // A pushpin on the map showing our location
        private Pushpin myLocationPushpin;


        // Constructor
        public MainPage()
        {
            InitializeComponent();
            InitializeGPS();
        }


        // Initialize the GPS sensor
        private void InitializeGPS()
        {
            if (gps == null)
            {
                gps = new GeoCoordinateWatcher(GeoPositionAccuracy.High);

                // Register for position change events (i.e., when we move) and 
                // status changed events
                gps.PositionChanged +=
                  new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>(watcher_PositionChanged);
                gps.StatusChanged +=
                  new EventHandler<GeoPositionStatusChangedEventArgs>(watcher_StatusChanged);
            }

            // Start the GPS sensor
            gps.Start();

            if (gps.Permission == GeoPositionPermission.Denied)
            {
                //Location services is disable on the phone. 
                this.buttonGPS.Background = new SolidColorBrush(Colors.Red);
            }
        }


        // Receive location updates when we move
        void watcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            mostRecentPosition = e.Position.Location;

            // Show the current location coordinates; longitude and latitude
            this.textBlockGPS.Text = "My location: " + mostRecentPosition.Longitude.ToString("0.000000") + " long, " +
                 mostRecentPosition.Latitude.ToString("0.000000") + " lat, ";
        }

        // The status of the GPS changes: either it's ready, or notifies
        // us about malfunction
        void watcher_StatusChanged(object sender, GeoPositionStatusChangedEventArgs e)
        {
            if (e.Status == GeoPositionStatus.Ready)
            {
                this.buttonGPS.Background = new SolidColorBrush(Colors.Green);
            }
            else if (e.Status == GeoPositionStatus.Disabled || e.Status == GeoPositionStatus.NoData)
            {
                this.buttonGPS.Background = new SolidColorBrush(Colors.Red);
            }
        }




        void pin_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            // When touching the pin, change the background color to yellow
            Pushpin pin = (Pushpin)sender;
            pin.Background = new SolidColorBrush(Colors.Yellow);
        }


        private void buttonNavigateMap_Click(object sender, RoutedEventArgs e)
        {
            // Go to the UofC campus
            this.map1.Center = new System.Device.Location.GeoCoordinate(51.07937094724242, -114.13043975830078);

            // Change the zoom level
            this.map1.ZoomLevel = 15;
        }


        private void buttonAerial_Click(object sender, RoutedEventArgs e)
        {
            // Show the aerial (i.e., satellite) view
            this.map1.Mode = new Microsoft.Phone.Controls.Maps.AerialMode();
        }


        private void buttonGPS_Click(object sender, RoutedEventArgs e)
        {
            // Set the center of the map to the current GPS location
            this.map1.Center = this.mostRecentPosition;
            this.map1.ZoomLevel = 16;

            // Create/update the location pushpin
            if (this.myLocationPushpin == null)
            {
                this.myLocationPushpin = new Pushpin();
                this.myLocationPushpin.Content = "My location";
                this.myLocationPushpin.Background = new SolidColorBrush(Colors.Blue);
                this.myLocationPushpin.Foreground = new SolidColorBrush(Colors.White);
                this.map1.Children.Add(this.myLocationPushpin);

                ScaleTransform transform = new ScaleTransform();
                this.myLocationPushpin.RenderTransform = transform;
                this.myLocationPushpin.RenderTransformOrigin = new System.Windows.Point(0, 1.0);

                // We're also animating the size of the pushpin
                DoubleAnimation animationX = new DoubleAnimation();
                animationX.To = 1.2;
                animationX.AutoReverse = true;
                animationX.RepeatBehavior = RepeatBehavior.Forever;

                DoubleAnimation animationY = new DoubleAnimation();
                animationY.To = 1.2;
                animationY.AutoReverse = true;
                animationY.RepeatBehavior = RepeatBehavior.Forever;

                SineEase ease = new SineEase();
                animationX.EasingFunction = ease;
                animationY.EasingFunction = ease;

                Storyboard storyboard = new Storyboard();
                Storyboard.SetTarget(animationY, transform);
                Storyboard.SetTargetProperty(animationY, new PropertyPath(ScaleTransform.ScaleYProperty));
                storyboard.Children.Add(animationY);
                Storyboard.SetTarget(animationX, transform);
                Storyboard.SetTargetProperty(animationX, new PropertyPath(ScaleTransform.ScaleXProperty));
                storyboard.Children.Add(animationX);
                storyboard.Begin();
            }

            this.myLocationPushpin.Location = this.mostRecentPosition;
        }


        private void map1_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (addPushpin == true)
            {
                // Create a new pin
                Pushpin pin = new Pushpin();

                // set the location of the pushpin to be the point where we are touching the map
                GeoCoordinate pinPosition = this.map1.ViewportPointToLocation(e.GetPosition(this.map1));
                pin.Location = pinPosition;
                pin.Background = new SolidColorBrush(Colors.Orange);
                pin.Foreground = new SolidColorBrush(Colors.Black);

                // Register for events when touching any of the pushpins
                pin.MouseLeftButtonDown += new MouseButtonEventHandler(pin_MouseLeftButtonDown);
                this.map1.Children.Add(pin);

                // as the label of the pin, show the distance (in meters) to our current
                // GPS location
                pin.Content = pinPosition.GetDistanceTo(mostRecentPosition).ToString("0.0") + " m distance";
                addPushpin = false;
            }
        }



        bool addPushpin = false;

        // Set a pushpin with the next click onto te map
        private void buttonAddPin_Click(object sender, RoutedEventArgs e)
        {
            addPushpin = true;
        }
    }
}